print 'Running post-install...'

import sys,os

if ('install' in sys.argv) and ('--help' not in sys.argv) and ('--help-commands' not in sys.argv):
    import wx_py

    d = wx_py.__path__[0]

    if sys.platform == 'win32':
        import pythoncom
        from win32com.shell import shell, shellcon
        shortcut = pythoncom.CoCreateInstance (
          shell.CLSID_ShellLink,
          None,
          pythoncom.CLSCTX_INPROC_SERVER,
          shell.IID_IShellLink
        )
        shortcut.SetPath (os.path.join(d,'PySlices.py'))
        shortcut.SetDescription ("PySlices "+str(wx_py.version.VERSION))
        shortcut.SetIconLocation (d, 0)
        
        desktop_path = shell.SHGetFolderPath (0, shellcon.CSIDL_DESKTOP, 0, 0)
        persist_file = shortcut.QueryInterface (pythoncom.IID_IPersistFile)
        persist_file.Save (os.path.join (desktop_path, "PySlices.lnk"), 0)
    elif sys.platform == 'linux2':
        fid = open(os.path.join(os.path.expanduser('~'),'Desktop','PySlices.desktop'),'w')
        fid.write('''#!/usr/bin/env xdg-open
[Desktop Entry]
Name=PySlices
Comment=None
Exec=python -m wx_py.PySlices
Icon='''+os.path.join(d,'PySlices.png')+'''
Terminal=False
Type=Application
Categories=Development;IDE
StartupNotify=true
MimeType=text/x-python;
GenericName=PySlices
''')
        fid.close()
    elif sys.platform == 'darwin':
        fid = open(os.path.join(os.path.expanduser('~'),'Desktop','PySlices.command'),'w')
        fid.write('''#!/bin/env sh
python -m wx_py.PySlices\n''')
        fid.close()



"""The py package, formerly the PyCrust package."""

__author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: __init__.py 63480 2010-02-14 05:34:39Z RD $"
__revision__ = "$Revision: 63480 $"[11:-2]

import buffer
import crust
import crustslices
import dispatcher
import document
import editor
import editwindow
import filling
import frame
import images
import interpreter
import introspect
import pseudo
import shell
import sliceshell
import version



commands = """
import sympy

try:
    pass#import numpy as np
except:
    pass

try:
    if set==0: pass # stupid construct to avoid printing anything
except NameError:
    import sets.Set as set

SYMPYSL_Integral_ = sympy.integrate
SYMPYSL_NarySummation_ = sympy.sum
SYMPYSL_PartialDifferential_ = sympy.diff
SYMPYSL_Infinity_ = sympy.oo
SYMPYSL_GreekSmallLetterPi_ = sympy.pi
SYMPYSL_DoublestruckItalicSmallI_ = sympy.I
SYMPYSL_DoublestruckItalicSmallJ_ = sympy.I
SYMPYSL_DoublestruckItalicSmallE_ = sympy.E
SYMPYSL_SquareRoot_ = sympy.sqrt
SYMPYSL_CubeRoot_  = lambda x: x**(sympy.Rational(1,3))
SYMPYSL_FourthRoot_ = lambda x: x**(sympy.Rational(1,4))

def __Union__(x,y):
    return set(x).union(set(y))
    
def __Intersection__(x,y):
    return set(x).intersection(set(y))

def __FractionSlash__(x,y):
    if x.__class__ in [int,long,str] and \\
       y.__class__ in [int,long,str]:
        return sympy.Rational(x,y)
    else:
        return x/y

__DivisionSign__ = __FractionSlash__

def __DotOperator__(a,b):
    if a.__class__ == sympy.Matrix and b.__class__ == sympy.Matrix:
        return a.dot(a)
    elif hasattr(a,'__iter__') and hasattr(b,'__iter__'):
        doListVersion=True
        try:
            np
        except NameError:
            pass
        else:
            if hasattr(np,'__package__'):
                if np.__package__ == 'numpy':
                    doListVersion=False
        if doListVersion:
            return sum([a[i]*b[i] for i in range(len(a))])
        else:
            return np.dot(a,b)
    else:
        return sympy.Symbol('__DotOperator__')(a,b)
    
# Do this at the end...
from sympy import *
del(sum) # sum should be the standard sum function, not sympy sum...
"""



#!/usr/bin/env python

from distutils.core import setup

setup( name = 'wx_py',
       version = '0.9.8.5c',
       description = 'Py Suite including PyCrust and a revamped version, PySlices',
       author = "David Mashburn / Patrick O'Brian",
       author_email = 'david.n.mashburn@gmail.com',
       url = 'http://www.wxpython.org/py.php',
       scripts = ['postinstall.py'],
       packages = ['wx_py'],
     )



# With so many values to index, I will have to automatically look for anything of the form SYMPYSL_*_
#   and then use dictionary indexing to find the result for any reasonable speed
# See MakeUnicodeDataFile.pyslices for some prototype file tools...
from wx_py.symbolConversionTable import unicodeConversionTable
import os
import ast
#from wx_py import symbolConversionNew

nameAddOn = 'SYMPYSL_'

precedence2astName = {
'not' : ast.Not, # special for the not character (unary)
'or' : ast.Or,
'and': ast.And,
'==' : ast.Eq,
'|'  : ast.BitOr,
'^'  : ast.BitXor,
'&'  : ast.BitAnd,
'<<' : ast.RShift,
'+'  : ast.Add, # Can also be unary...
'*'  : ast.Mult,
'**' : ast.Pow
}

testNames=False

allNames = []
for i in unicodeConversionTable:
    newNames = [i[1]]+i[2]
    if testNames:
        for j in newNames:
            if j in allNames:
                print 'Error with alias '+j+' at symbol number '+hex(i[0])
    allNames += newNames


allCatagories = []
for i in unicodeConversionTable:
    if i[3] not in allCatagories:
        allCatagories.append(i[3])

knownCatagories = ['Ascii','SpecialEscapeCharacter','Display',
                   'SimpleSubstitution', 'Symbol', 'InfixOperator']
# 'Ascii' (ord<128) is ignored because it is already handled properly by the interpreter...
# 'SpecialEscapeCharacter' is reserved for the special inputting character
# 'Display' is able to be saved and viewed, but will not convert to valid code
# 'Symbol' is the most general catagory: it can be a letter, prefix operator,
#    or other symbol (like infinity) that can hold either a value or be a function...
# 'InfixOperator' is a unique catagory for any operator that requires the use of
#    an ast substitution to work properly.  This type of symbol MUST define
#    a replacement operator (one of the keys to precedence2astName)
#    to determine the order of operations.
#    --- On a side note, I still need to update the ast modifier to allow
#        equality operators separate from simple binary operators...

# Also, when this file gets finished, one of the functions should be:
#    ConvertTableToDictionaries.  This will create a file separate (*Data.py)
#    that can be imported instead of *Table.py to avoid re-processing *Table.py
#    on every import.

for i in allCatagories:
    if i not in knownCatagories:
        print 'Catagory',i,'is an unknown catagory and will not be handled properly...'

##########################################################
# Divide the table up into catagories...
##########################################################
AsciiTable = [i for i in unicodeConversionTable if i[3]=='Ascii']
SpecialEscapeCharacterTable = [i for i in unicodeConversionTable if i[3]=='SpecialEscapeCharacter']
DisplayTable = [i for i in unicodeConversionTable if i[3]=='Display']
SimpleSubstitutionTable = [i for i in unicodeConversionTable if i[3]=='SimpleSubstitution']
SymbolTable = [i for i in unicodeConversionTable if i[3]=='Symbol']
InfixOperatorTable = [i for i in unicodeConversionTable if i[3]=='InfixOperator']

ESC_SYMBOL = unichr(0x0022ee).encode('utf-8')
nameAddOn = 'SYMPYSL_'

# ##################################################
# Auto-generate the dictionaries from the tables...
# ##################################################

# Skip Ascii

if len(SpecialEscapeCharacterTable) > 1:
    print 'Should only be one escape character!!!'

infixOperatorNames = [i[1] for i in InfixOperatorTable]
for i in InfixOperatorTable:
    infixOperatorNames += i[2]

NameToInfixAstSubstitute = dict([ [i[1],i[4]] for i in InfixOperatorTable ])
for i in InfixOperatorTable:
    for j in i[2]:
        NameToInfixAstSubstitute[j] = i[4]

# For conversion before saving
ToName = dict([ [unichr(i[0]),i[1]] for i in
  DisplayTable + SimpleSubstitutionTable + SymbolTable + InfixOperatorTable ])
ToName[unichr(SpecialEscapeCharacterTable[0][0])] = \
                                       SpecialEscapeCharacterTable[0][1]

# For loading and conversion of output from interpreter
FromName = dict([ [i[1],unichr(i[0]).encode('utf-8')] for i in
  DisplayTable + SimpleSubstitutionTable + SymbolTable + InfixOperatorTable ])
FromName[SpecialEscapeCharacterTable[0][1]] = \
               unichr(SpecialEscapeCharacterTable[0][0]).encode('utf-8')

for i in DisplayTable+SimpleSubstitutionTable+SymbolTable+InfixOperatorTable:
    for alias in i[2]:
        FromName[alias]=FromName[i[1]] # alias points to same thing as the name...


# For conversion before passing to interpreter
ToInterpreter = dict(
  [[ unichr(i[0]),nameAddOn+i[1]+'_'] for i in SymbolTable+InfixOperatorTable] + \
  [[ unichr(i[0]),i[4]] for i in SimpleSubstitutionTable]
                    )

def WriteDicts2File(directory=None):
    if directory == None:
        directory = os.getcwd()
    fid = open(os.path.join(directory,'symbolConversionDicts.py'),'w')
    
    fid.write('# This file is autogenerated from the Table...\n')
    fid.write('# Make modifications to symbolConversionTable.py, then \n')
    fid.write('#   regenerate this file with symbolConversionDictWriter.py \n')
    fid.write('\n')
    
    # InfixOperatorNames
    fid.write('infixOperatorNames = ')
    fid.write(str(infixOperatorNames).replace(', ' , ',\n').replace('[','[\n'))
    fid.write('\n\n')
    
    # NameToInfixAstSubstitute
    fid.write('NameToInfixAstSubstitute = ')
    fid.write(str(NameToInfixAstSubstitute).replace(', ' , ',\n').replace('{','{\n'))
    fid.write('\n\n')
    
    # ToName
    fid.write('ToName = ')
    fid.write(str(ToName).replace(', ' , ',\n').replace('{','{\n'))
    fid.write('\n\n')
    
    # FromName
    fid.write('FromName = ')
    fid.write(str(FromName).replace(', ' , ',\n').replace('{','{\n'))
    fid.write('\n\n')
    
    # ToInterpreter
    fid.write('ToInterpreter = ')
    fid.write(str(ToInterpreter).replace(', ' , ',\n').replace('{','{\n'))
    fid.write('\n\n')
    
    fid.close

if __name__ == '__main__':
    WriteDicts2File()

def QuickTest():
    # A quick test of the modified parts of the symbolConversionTable:
    from wx_py import symbolConversionTable
    for i in symbolConversionTable.unicodeConversionTable:
        if i[2]!=[] or i[3] not in ['Display','Ascii']:
            print i



"""PySlices combines the slices and filling into one control."""

__author__ = "David N. Mashburn <david.n.mashburn@gmail.com> / "
__author__ += "Patrick K. O'Brien <pobrien@orbtech.com>"
__cvsid__ = "$Id: crustslices.py 44235 2007-01-17 23:05:14Z RD $"
__revision__ = "$Revision: 44235 $"[11:-2]

import wx

import os
import pprint
import re
import sys

import dispatcher
import crust
import document
import editwindow
import editor
from filling import Filling
import frame
from sliceshell import SlicesShell
from version import VERSION


class CrustSlices(crust.Crust):
    """Slices based on SplitterWindow."""

    name = 'Slices'
    revision = __revision__
    sashoffset = 300

    def __init__(self, parent, id=-1, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=wx.SP_3D|wx.SP_LIVE_UPDATE,
                 name='Slices Window', rootObject=None, rootLabel=None,
                 rootIsNamespace=True, intro='', locals=None,
                 InterpClass=None,
                 startupScript=None, execStartupScript=True,
                 showPySlicesTutorial=True,
                 enableShellMode=False, hideFoldingMargin=False,
                 *args, **kwds):
        """Create CrustSlices instance."""
        wx.SplitterWindow.__init__(self, parent, id, pos, size, style, name)

        # Turn off the tab-traversal style that is automatically
        # turned on by wx.SplitterWindow.  We do this because on
        # Windows the event for Ctrl-Enter is stolen and used as a
        # navigation key, but the SlicesShell window uses it to insert lines.
        style = self.GetWindowStyle()
        self.SetWindowStyle(style & ~wx.TAB_TRAVERSAL)
        
        self.sliceshell = SlicesShell(parent=self, introText=intro,
                                 locals=locals, InterpClass=InterpClass,
                                 startupScript=startupScript,
                                 execStartupScript=execStartupScript,
                                 showPySlicesTutorial=showPySlicesTutorial,
                                 enableShellMode=enableShellMode,
                                 hideFoldingMargin=hideFoldingMargin,
                                 *args, **kwds)
        
        self.editor = self.sliceshell
        self.shell = self.sliceshell
        if rootObject is None:
            rootObject = self.sliceshell.interp.locals
        self.notebook = wx.Notebook(parent=self, id=-1)
        self.sliceshell.interp.locals['notebook'] = self.notebook
        self.filling = Filling(parent=self.notebook,
                               rootObject=rootObject,
                               rootLabel=rootLabel,
                               rootIsNamespace=rootIsNamespace)
        # Add 'filling' to the interpreter's locals.
        self.sliceshell.interp.locals['filling'] = self.filling
        self.notebook.AddPage(page=self.filling, text='Namespace', select=True)
        
        self.display = crust.Display(parent=self.notebook)
        self.notebook.AddPage(page=self.display, text='Display')
        # Add 'pp' (pretty print) to the interpreter's locals.
        self.sliceshell.interp.locals['pp'] = self.display.setItem
        self.display.nbTab = self.notebook.GetPageCount()-1
        
        self.calltip = crust.Calltip(parent=self.notebook,ShellClassName='SlicesShell')
        self.notebook.AddPage(page=self.calltip, text='Calltip')
        
        self.sessionlisting = crust.SessionListing(parent=self.notebook,ShellClassName='SlicesShell')
        self.notebook.AddPage(page=self.sessionlisting, text='History')
        
        self.dispatcherlisting = crust.DispatcherListing(parent=self.notebook)
        self.notebook.AddPage(page=self.dispatcherlisting, text='Dispatcher')

        
        # Initialize in an unsplit mode, and check later after loading
        # settings if we should split or not.
        self.sliceshell.Hide()
        self.notebook.Hide()
        self.Initialize(self.sliceshell)
        self._shouldsplit = True
        wx.CallAfter(self._CheckShouldSplit)
        self.SetMinimumPaneSize(100)

        self.Bind(wx.EVT_SIZE, self.SplitterOnSize)
        self.Bind(wx.EVT_SPLITTER_SASH_POS_CHANGED, self.OnChanged)
        self.Bind(wx.EVT_SPLITTER_DCLICK, self.OnSashDClick)

class CrustSlicesFrame(crust.CrustFrame):
    """Frame containing all the PySlices components."""

    name = 'SliceFrame'
    revision = __revision__


    def __init__(self, parent=None, id=-1, title='PySlices',
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=wx.DEFAULT_FRAME_STYLE,
                 rootObject=None, rootLabel=None, rootIsNamespace=True,
                 locals=None, InterpClass=None,
                 config=None, dataDir=None, filename=None,
                 *args, **kwds):
        """Create CrustFrame instance."""
        frame.Frame.__init__(self, parent, id, title, pos, size, style,
                             shellName='PySlices')
        frame.ShellFrameMixin.__init__(self, config, dataDir)
        
        if size == wx.DefaultSize:
            self.SetSize((800, 600))
        
        intro = 'PySlices %s - The Flakiest Python Shell... Cut up!' % VERSION
        
        self.SetStatusText(intro.replace('\n', ', '))
        self.crust = CrustSlices(parent=self, intro=intro,
                                 rootObject=rootObject,
                                 rootLabel=rootLabel,
                                 rootIsNamespace=rootIsNamespace,
                                 locals=locals,
                                 InterpClass=InterpClass,
                                 startupScript=self.startupScript,
                                 execStartupScript=self.execStartupScript,
                                 showPySlicesTutorial=self.showPySlicesTutorial,
                                 enableShellMode=self.enableShellMode,
                                 hideFoldingMargin=self.hideFoldingMargin,
                                 *args, **kwds)
        self.sliceshell = self.crust.sliceshell
        self.buffer = self.sliceshell.buffer
        # Override the filling so that status messages go to the status bar.
        self.crust.filling.tree.setStatusText = self.SetStatusText
        
        # Override the shell so that status messages go to the status bar.
        self.sliceshell.setStatusText = self.SetStatusText
        
        self.sliceshell.SetFocus()
        self.LoadSettings()
        
        self.currentDirectory = os.getcwd()
        
        if filename!=None:
            self.bufferOpen(filename)
        
        self.Bind(wx.EVT_IDLE, self.OnIdle)

    def OnClose(self, event):
        """Event handler for closing."""
        self.bufferClose()
    
    def OnAbout(self, event):
        """Display an About window."""
        title = 'About PySlices'
        text = 'PySlices %s\n\n' % VERSION + \
               'Yet another Python shell, only flakier.\n\n' + \
               'Half-baked by Patrick K. O\'Brien,\n' + \
               'the other half is still in the oven.\n\n' + \
               'Shell Revision: %s\n' % self.sliceshell.revision + \
               'Interpreter Revision: %s\n\n' % self.sliceshell.interp.revision + \
               'Platform: %s\n' % sys.platform + \
               'Python Version: %s\n' % sys.version.split()[0] + \
               'wxPython Version: %s\n' % wx.VERSION_STRING + \
               ('\t(%s)\n' % ", ".join(wx.PlatformInfo[1:]))
        dialog = wx.MessageDialog(self, text, title,
                                  wx.OK | wx.ICON_INFORMATION)
        dialog.ShowModal()
        dialog.Destroy()

    def OnEnableShellMode(self,event):
        """Change between Slices Mode and Shell Mode"""
        frame.Frame.OnEnableShellMode(self,event)
        self.sliceshell.ToggleShellMode(self.enableShellMode)
    
    def OnHideFoldingMargin(self,event):
        """Change between Slices Mode and Shell Mode"""
        frame.Frame.OnHideFoldingMargin(self,event)
        self.sliceshell.ToggleFoldingMargin(self.hideFoldingMargin)

    # Stolen Straight from editor.EditorFrame
    # Modified a little... :)
    # ||
    # \/
    def OnIdle(self, event):
        """Event handler for idle time."""
        self._updateTitle()
        event.Skip()
    
    def _updateTitle(self):
        """Show current title information."""
        title = self.GetTitle()
        if self.bufferHasChanged():
            if title.startswith('* '):
                pass
            else:
                self.SetTitle('* ' + title)
        else:
            if title.startswith('* '):
                self.SetTitle(title[2:])
    
    def hasBuffer(self):
        """Return True if there is a current buffer."""
        if self.buffer:
            return True
        else:
            return False

    def bufferClose(self):
        """Close buffer."""
        if self.buffer.hasChanged():
            cancel = self.bufferSuggestSave()
            if cancel:
                #event.Veto()
                return cancel
        self.SaveSettings()
        self.crust.sliceshell.destroy()
        self.bufferDestroy()
        self.Destroy()
        
        return False

    def bufferCreate(self, filename=None):
        """Create new buffer."""
        self.bufferDestroy()
        buffer = Buffer()
        self.panel = panel = wx.Panel(parent=self, id=-1)
        panel.Bind (wx.EVT_ERASE_BACKGROUND, lambda x: x)        
        editor = Editor(parent=panel)
        panel.editor = editor
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(editor.window, 1, wx.EXPAND)
        panel.SetSizer(sizer)
        panel.SetAutoLayout(True)
        sizer.Layout()
        buffer.addEditor(editor)
        buffer.open(filename)
        self.setEditor(editor)
        self.editor.setFocus()
        self.SendSizeEvent()
        

    def bufferDestroy(self):
        """Destroy the current buffer."""
        if self.buffer:
            self.editor = None
            self.buffer = None


    def bufferHasChanged(self):
        """Return True if buffer has changed since last save."""
        if self.buffer:
            return self.buffer.hasChanged()
        else:
            return False

    def bufferNew(self):
        """Create new buffer."""
        cancel = self.bufferSuggestSave()
        if cancel:
            return cancel
        self.sliceshell.clear()
        self.SetTitle( 'PySlices')
        self.sliceshell.NeedsCheckForSave=False
        self.sliceshell.SetSavePoint()
        self.buffer.doc = document.Document()
        self.buffer.name = 'This shell'
        self.buffer.modulename = self.buffer.doc.filebase
        #self.bufferCreate()
        cancel = False
        return cancel

    def bufferOpen(self,file=None):
        """Open file in buffer."""
        if self.bufferHasChanged():
            cancel = self.bufferSuggestSave()
            if cancel:
                return cancel
        
        if file==None:
            file=wx.FileSelector('Open a PySlices File',
                                 wildcard='*.pyslices',
                                 default_path=self.currentDirectory)
        if file!=None and file!=u'':
            fid=open(file,'r')
            self.sliceshell.LoadPySlicesFile(fid)
            fid.close()
            self.currentDirectory = os.path.split(file)[0]
            self.SetTitle( os.path.split(file)[1] + ' - PySlices')
            self.sliceshell.NeedsCheckForSave=False
            self.sliceshell.SetSavePoint()
            self.buffer.doc = document.Document(file)
            self.buffer.name = self.buffer.doc.filename
            self.buffer.modulename = self.buffer.doc.filebase
            self.sliceshell.ScrollToLine(0)
        return
    
##     def bufferPrint(self):
##         """Print buffer."""
##         pass

##     def bufferRevert(self):
##         """Revert buffer to version of file on disk."""
##         pass
    
    # was self.buffer.save(self): # """Save buffer."""
    def simpleSave(self,confirmed=False):
        filepath = self.buffer.doc.filepath
        self.buffer.confirmed = confirmed
        if not filepath:
            return  # XXX Get filename
        if not os.path.exists(filepath):
            self.buffer.confirmed = True
        if not self.buffer.confirmed:
            self.buffer.confirmed = self.buffer.overwriteConfirm(filepath)
        if self.buffer.confirmed:
            try:
                fid = open(filepath, 'wb')
                self.sliceshell.SavePySlicesFile(fid)
            finally:
                if fid:
                    fid.close()
            self.sliceshell.SetSavePoint()
            self.SetTitle( os.path.split(filepath)[1] + ' - PySlices')
            self.sliceshell.NeedsCheckForSave=False
    
    def bufferSave(self):
        """Save buffer to its file."""
        if self.buffer.doc.filepath:
            # self.buffer.save()
            self.simpleSave(confirmed=True)
            cancel = False
        else:
            cancel = self.bufferSaveAs()
        return cancel

    def bufferSaveAs(self):
        """Save buffer to a new filename."""
        if self.bufferHasChanged() and self.buffer.doc.filepath:
            cancel = self.bufferSuggestSave()
            if cancel:
                return cancel
        filedir = ''
        if self.buffer and self.buffer.doc.filedir:
            filedir = self.buffer.doc.filedir
        result = editor.saveSingle(title='Save PySlices File',directory=filedir,
                                   wildcard='PySlices Files (*.pyslices)|*.pyslices')
        if result.path not in ['',None]:
            if result.path[-9:]!=".pyslices":
                result.path+=".pyslices"
            
            self.buffer.doc = document.Document(result.path)
            self.buffer.name = self.buffer.doc.filename
            self.buffer.modulename = self.buffer.doc.filebase
            self.simpleSave(confirmed=True) # allow overwrite
            cancel = False
        else:
            cancel = True
        return cancel
    
    def bufferSaveACopy(self):
        """Save buffer to a new filename."""
        filedir = ''
        if self.buffer and self.buffer.doc.filedir:
            filedir = self.buffer.doc.filedir
        result = editor.saveSingle(title='Save a Copy of PySlices File',directory=filedir,
                                   wildcard='PySlices Files (*.pyslices)|*.pyslices')
        
        if result.path not in ['',None]:
            if result.path[-9:]!=".pyslices":
                result.path+=".pyslices"
            
            # if not os.path.exists(result.path):
            try: # Allow overwrite...
                fid = open(result.path, 'wb')
                self.sliceshell.SavePySlicesFile(fid)
            finally:
                if fid:
                    fid.close()
                
            cancel = False
        else:
            cancel = True
        return cancel
    
    def bufferSuggestSave(self):
        """Suggest saving changes.  Return True if user selected Cancel."""
        result = editor.messageDialog(parent=None,
                               message='%s has changed.\n'
                                       'Would you like to save it first'
                                       '?' % self.buffer.name,
                               title='Save current file?',
                               )
                               #style=wx.YES_NO | wx.CANCEL | wx.NO_DEFAULT |
                               #      wx.CENTRE | wx.ICON_QUESTION )
        if result.positive:
            cancel = self.bufferSave()
        else:
            cancel = result.text == 'Cancel'
        return cancel

    def updateNamespace(self):
        """Update the buffer namespace for autocompletion and calltips."""
        if self.buffer.updateNamespace():
            self.SetStatusText('Namespace updated')
        else:
            self.SetStatusText('Error executing, unable to update namespace')
